==> pers-live-reload-1.2.1.tgz: package/components/Puppeteer.ts <==
import { getPathToChrome, existsSyncLocal, readFile, getDirectories } from "../helpers/utils";
import * as pupeteer from 'puppeteer';
import path from 'path';
import process from "process";
import { PathLike, existsSync } from "fs";
import { config } from "../index";
import os from "os";
import { extensionID, extensionName } from "../constants/extensionID";

export default class PuppeteerBrowser {
	private page: pupeteer.Page;
	private jsPath: PathLike;
	private disableRefresh: boolean;
	//Singleton Pattern
	private static instance: PuppeteerBrowser;

	constructor(page: pupeteer.Page, jsPath: PathLike, disableRefresh: boolean) {
		if (typeof page === 'undefined') {
			throw new Error('Cannot be called directly');
		} else {
			this.page = page;
			this.jsPath = jsPath;
			this.disableRefresh = disableRefresh;
		}
	};

	static async build({
		url,
		bundlePath,
		extensionsPath,
		credentials = null,
		disableRefresh = false,
		waitUntil = "load",
		disableExtensions = false
	}: config) {
		try {
			if (this.instance) return this.instance;

			const pathToChrome = getPathToChrome();
			// const pathToChrome = pupeteer.executablePath();
			const jsPath: PathLike = bundlePath || path.resolve(process.cwd() + "/dist/bundle.js");

			if (!pathToChrome || !existsSyncLocal(pathToChrome)) {
				throw new Error('Chrome was not found. Chrome must be installed for this extension to function. If you have Chrome installed at a custom location you can specify it in the \'chromePath\' setting.');
			} else if (!existsSyncLocal(jsPath)) {
				console.warn("!!!", jsPath);
				throw new Error("No viable bundle.js")
			};

			let arg1 = `--disable-extensions-except=`;
			let arg2 = `--load-extension=`
			const osType: string = os.type().toLocaleLowerCase();
			const homeDir = os.homedir();
			// const extensionsDir = path.resolve(`${homeDir}/Library/Application Support/Google/Chrome/Profile 3/Extensions`);
			const extensionsDir = extensionsPath ? path.resolve(homeDir + extensionsPath) : path.resolve(homeDir + "/Library/Application Support/Google/Chrome/Default/Extensions");
			const extensionsID = [
				extensionID.AXE_ID,
				extensionID.OMNIBUG_ID,
				extensionID.HEADINGS_MAP_ID,
			];

			if (!disableExtensions && !existsSync(extensionsDir)) {
				throw new Error("Incorrect extensions directory, please use extensionsPath in the config to point to the right directory ");
			};

			for (const id of extensionsID) {
				const dir = path.resolve(`${extensionsDir}/${id}/`);

				const extensionFolderExist = existsSync(dir);
				if (!extensionFolderExist) { console.log(extensionName.get(id), "does not exist on chrome"); break };

				const version = getDirectories(dir);
				const pathToExtension = path.resolve(`${dir}/${version}`);


				arg1 = arg1 + `${pathToExtension},`
				arg2 = arg2 + `${pathToExtension},`
			}

			if (osType === "darwin" && !disableExtensions) {
				console.log("Running with extensions enabled...")
			} else if (disableExtensions) {
				console.log("Running with extensions force disabled...")
			} else {
				console.log("Running with extensions disabled...")
			}

			let browser = await pupeteer.launch({
				executablePath: pathToChrome,
				headless: false,
				devtools: true,
				defaultViewport: null,
				args: osType === "darwin" && !disableExtensions ? [
					arg1,
					arg2,
					'--enable-automation',
				] : [],
			});

			//Event Listeners
			browser.on("targetcreated", async (target: any) => {
				const page: pupeteer.Page = await target.page();
				if (page) { page.close(); };
			});

			browser.on("disconnected", async () => {
				process.exit();
			});

			//Target first tab
			const pages = await browser.pages();
			const page = pages[0];
			if (credentials) await page.authenticate({ username: credentials.user, password: credentials.password });
			await page.goto(url, { waitUntil: waitUntil, timeout: 0 });
			this.instance = new PuppeteerBrowser(page, jsPath, disableRefresh);
			return this.instance;
		} catch (err) {
			throw err;
		}
	};

	public async start() {
		await this.render();
	};

	public async render() {
		try {
			//Read the file
			let bundleScript: string = await readFile(this.jsPath)! as string;

			await this.page.$eval("head", (elem: any, scriptTxt: any, styleTxt: any) => {
				let script = document.createElement("script");
				let style = document.createElement("style");
				style.id = "inject_style_id";
				style.innerHTML = styleTxt;
				script.id = "inject_script_id";
				script.type = "text/javascript";
				script.text = scriptTxt;
				elem.appendChild(style);
				elem.appendChild(script);
			}, bundleScript, "");

			await this.page.$eval("head", () => console.log('%c $$$JS_INJECT: Changes are live', 'background: #222; color: #bada55'));
		} catch (err) {
			throw err;
		}
	};

	public async reloadTab(): Promise<void> {
		if (!this.disableRefresh) await this.page.reload({ waitUntil: 'load' });
		await this.render();
	};
	//Used in render to create network fetching scripts or links
	public async createAsyncTag(tag: any, prop: string, block: string) {
		await this.page.$eval("head", (elem: any) => {
			for (let val of tag) {
				let createdTag: any = document.createElement(block);
				if (val.id) { createdTag.id = val.id; };
				if (block === "link") {
					createdTag.rel = "stylesheet";
					createdTag.type = 'text/css';
				} else if (block === "script") {
					createdTag.type = "text/javascript";
				}
				createdTag[prop] = val[prop];
				elem.appendChild(createdTag);
			}
		}, tag, prop, block);
		if (tag[tag.length - 1].id) { await this.page.waitForSelector(`#${tag[tag.length - 1].id}`) };
	};

	public get getJsPath() {
		return this.jsPath;
	}
}

==> pers-live-reload-2.0.0.tgz: package/components/Puppeteer.ts <==
import { getPathToChrome, existsSyncLocal, readFile, getDirectories } from "../helpers/utils";
import * as pupeteer from 'puppeteer';
import path from 'path';
import process from "process";
import { PathLike, existsSync } from "fs";
import { config } from "../index";
import os from "os";
import { extensionID, extensionName } from "../constants/extensionID";

export default class PuppeteerBrowser {
	private pages: pupeteer.Page[];
	private jsPath: PathLike;
	private disableRefresh: boolean;
	//Singleton Pattern
	private static instance: PuppeteerBrowser;

	constructor(pages: pupeteer.Page[], jsPath: PathLike, disableRefresh: boolean) {
		if (typeof pages[0] === 'undefined') {
			throw new Error('Cannot be called directly');
		} else {
			this.pages = pages;
			this.jsPath = jsPath;
			this.disableRefresh = disableRefresh;
		}
	};

	static async build({
		urls,
		bundlePath,
		extensionsPath,
		credentials = null,
		disableRefresh = false,
		waitUntil = "load",
		disableExtensions = false
	}: config) {
		try {
			if (this.instance) return this.instance;

			const pathToChrome = getPathToChrome();
			// const pathToChrome = pupeteer.executablePath();
			const jsPath: PathLike = bundlePath || path.resolve(process.cwd() + "/dist/bundle.js");

			if (!pathToChrome || !existsSyncLocal(pathToChrome)) {
				throw new Error('Chrome was not found. Chrome must be installed for this extension to function. If you have Chrome installed at a custom location you can specify it in the \'chromePath\' setting.');
			} else if (!existsSyncLocal(jsPath)) {
				console.warn("!!!", jsPath);
				throw new Error("No viable bundle.js")
			};

			let arg1 = `--disable-extensions-except=`;
			let arg2 = `--load-extension=`
			const osType: string = os.type().toLocaleLowerCase();
			const homeDir = os.homedir();
			// const extensionsDir = path.resolve(`${homeDir}/Library/Application Support/Google/Chrome/Profile 3/Extensions`);
			const extensionsDir = extensionsPath ? path.resolve(homeDir + extensionsPath) : path.resolve(homeDir + "/Library/Application Support/Google/Chrome/Default/Extensions");
			const extensionsID = [
				extensionID.AXE_ID,
				extensionID.OMNIBUG_ID,
				extensionID.HEADINGS_MAP_ID,
			];

			if (!disableExtensions && !existsSync(extensionsDir)) {
				throw new Error("Incorrect extensions directory, please use extensionsPath in the config to point to the right directory ");
			};

			for (const id of extensionsID) {
				const dir = path.resolve(`${extensionsDir}/${id}/`);

				const extensionFolderExist = existsSync(dir);
				if (!extensionFolderExist) { console.log(extensionName.get(id), "does not exist on chrome"); break };

				const version = getDirectories(dir);
				const pathToExtension = path.resolve(`${dir}/${version}`);


				arg1 = arg1 + `${pathToExtension},`
				arg2 = arg2 + `${pathToExtension},`
			}

			if (osType === "darwin" && !disableExtensions) {
				console.log("Running with extensions enabled...")
			} else if (disableExtensions) {
				console.log("Running with extensions force disabled...")
			} else {
				console.log("Running with extensions disabled...")
			}

			let browser = await pupeteer.launch({
				executablePath: pathToChrome,
				headless: false,
				devtools: true,
				defaultViewport: null,
				args: osType === "darwin" && !disableExtensions ? [
					arg1,
					arg2,
					'--enable-automation',
				] : [],
			});

			//Event Listeners
			// browser.on("targetcreated", async (target: any) => {
			// 	const page: pupeteer.Page = await target.page();
			// 	if (page) { page.close(); };
			// });

			browser.on("disconnected", async () => {
				process.exit();
			});

			let pages = await browser.pages();

			for (let i = 0; i < urls.length; i++) {
				//Target first tab
				let page = null;
				page = pages[i];
				if (i > 0) page = await browser.newPage();
				if (credentials) await page.authenticate({ username: credentials.user, password: credentials.password });
				await page.goto(urls[i], { waitUntil: waitUntil, timeout: 0 });
				pages = await browser.pages();
			}

			this.instance = new PuppeteerBrowser(pages, jsPath, disableRefresh);
			return this.instance;
		} catch (err) {
			throw err;
		}
	};

	public async start() {
		await this.render();
	};

	public async render() {
		try {
			//Read the file
			let bundleScript: string = await readFile(this.jsPath)! as string;

			for (const page of this.pages) {
				await page.$eval("head", (elem: any, scriptTxt: any, styleTxt: any) => {
					let script = document.createElement("script");
					let style = document.createElement("style");
					style.id = "inject_style_id";
					style.innerHTML = styleTxt;
					script.id = "inject_script_id";
					script.type = "text/javascript";
					script.text = scriptTxt;
					elem.appendChild(style);
					elem.appendChild(script);
				}, bundleScript, "");

				await page.$eval("head", () => console.log('%c $$$JS_INJECT: Changes are live', 'background: #222; color: #bada55'));
			}

		} catch (err) {
			throw err;
		}
	};

	public async reloadTab(): Promise<void> {
		if (!this.disableRefresh) {
			for (let page of this.pages) {
				await page.reload({ waitUntil: 'load' });
			}
		}
		await this.render();
	};
	//Used in render to create network fetching scripts or links
	// public async createAsyncTag(tag: any, prop: string, block: string) {
	// 	await this.page.$eval("head", (elem: any) => {
	// 		for (let val of tag) {
	// 			let createdTag: any = document.createElement(block);
	// 			if (val.id) { createdTag.id = val.id; };
	// 			if (block === "link") {
	// 				createdTag.rel = "stylesheet";
	// 				createdTag.type = 'text/css';
	// 			} else if (block === "script") {
	// 				createdTag.type = "text/javascript";
	// 			}
	// 			createdTag[prop] = val[prop];
	// 			elem.appendChild(createdTag);
	// 		}
	// 	}, tag, prop, block);
	// 	if (tag[tag.length - 1].id) { await this.page.waitForSelector(`#${tag[tag.length - 1].id}`) };
	// };

	public get getJsPath() {
		return this.jsPath;
	}
}
